@@ -15,6 +15,7 @@ from tensorflow.python.framework.dtypes import (
 import tensorflow_datasets as tfds
 from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import file_adapters
+from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import Version, version
 from tensorflow_datasets.core.download import DownloadManager
 
@@ -30,6 +31,11 @@ from tfdsio.utils import logger
 from tfdsio.io import (
     PipelineIO
 )
+try:
+    from datasets import Dataset, DatasetDict
+except ImportError:
+    Dataset = object
+    DatasetDict = object
 
 
 _has_logged_ds_info = set()
@@ -131,6 +137,9 @@ class BuilderConfig:
     stream_data: bool = True # Stream data if from GCS / FileLike without Downloading
     preprocessors: Optional[List[Callable]] = None
     preprocessor_kwargs: Optional[Dict[str, Any]] = None
+    
+    # Will re-process the dataset
+    rewrite_existing: bool = False
 
     def get_datasources(
         self, 
@@ -231,13 +240,16 @@ class TFDSDatasetBuilder(tfds.core.GeneratorBasedBuilder):
             config.data_dir = os.fspath(os.environ['TFDS_DIR'])
         elif os.environ.get('GCS_DIR', None):
             config.data_dir = os.fspath(os.environ['GCS_DIR'])
-
+        
         file_format = config.file_format or file_adapters.DEFAULT_FILE_FORMAT
         self._original_state = dict(data_dir = config.data_dir, config = config, version = config.version)
         self._builder_config: BuilderConfig = self._create_builder_config(config)
-        self.__dict__['name'] = self._builder_config.name
+        self.name = self._builder_config.name
         self._version = self._pick_version(config.version)
         self._data_dir_root, self._data_dir = self._build_data_dir(config)
+        if config.rewrite_existing and gfile.Exists(self._data_dir):
+            self._remove_existing_data_dir()
+        
         if gfile.Exists(self._data_dir):
             self.info.read_from_directory(self._data_dir)
             self._log_dataset_info()
@@ -248,14 +260,21 @@ class TFDSDatasetBuilder(tfds.core.GeneratorBasedBuilder):
             self._file_format = file_adapters.FileFormat(file_format)
             self.info.set_file_format(self._file_format)
 
-        except ValueError:
+        except ValueError as e:
             all_values = [f.value for f in file_adapters.FileFormat]
-            raise ValueError(f"{file_format} is not a valid format. Valid file formats: {all_values}")
+            raise ValueError(f"{file_format} is not a valid format. Valid file formats: {all_values}") from e
 
+    def _remove_existing_data_dir(self):
+        """
+        Removes a previously created dataset
+        if config.rewrite_existing is True
+        """
+        logger.warning(f"Removing and overwriting dataset {self.name} at {self._data_dir}")
+        gfile.DeleteRecursively(self._data_dir)
     
     def _log_dataset_info(self):
         msg = "--------------------------------------------\n"
-        msg += f"MLX Dataset: {self.info.full_name}\n"
+        msg += f"TFDSIO Dataset: {self.info.full_name}\n"
         msg += "Splits:\n"
         for split, details in self.info.splits.items():
             msg += f" - {split}: {details.num_examples} examples, {details.num_shards} shards\n"
@@ -370,7 +389,7 @@ class TFDSDatasetBuilder(tfds.core.GeneratorBasedBuilder):
             else:
                 for preprocessor in self.builder_config.preprocessors:
                     ex = preprocessor(idx = idx, data = ex, extra = extra, **self.get_preprocessor_kwargs(**kwargs))
-                    for (n, i) in self.map_to_features(preprocessor(ex), idx, has_preprocessor = True):
+                    for (n, i) in self.map_to_features(ex, idx, has_preprocessor = True):
                         yield n, i
                         idx += 1
 
@@ -399,8 +418,8 @@ class TFDSDatasetBuilder(tfds.core.GeneratorBasedBuilder):
         given_data_dir = config.data_dir
         builder_dir = self._relative_data_dir(with_version=False)
         version_dir = self._relative_data_dir(with_version=True)
-        default_data_dir = constants.get_default_data_dir(given_data_dir=given_data_dir)
-        all_data_dirs = constants.list_data_dirs(given_data_dir=given_data_dir)
+        default_data_dir = file_utils.get_default_data_dir(given_data_dir=given_data_dir)
+        all_data_dirs = file_utils.list_data_dirs(given_data_dir=given_data_dir)
         all_versions = set()
         requested_version_dirs = {}
         for data_dir_root in all_data_dirs:
@@ -466,6 +485,97 @@ class TFDSDatasetBuilder(tfds.core.GeneratorBasedBuilder):
         return DatasetInfo(builder = self, **self.builder_config.dataset_info)
 
 
+
+class HFDatasetBuilder(TFDSDatasetBuilder):
+
+    VERSION = None
+    RELEASE_NOTES: ClassVar[Dict[str, str]] = {}
+    SUPPORTED_VERSIONS = []
+    BUILDER_CONFIGS = []
+    MANUAL_DOWNLOAD_INSTRUCTIONS = None
+
+    def __init__(
+        self, 
+        *, 
+        dataset: Union[Dataset, DatasetDict],
+        config: Union[None, str, BuilderConfig] = None, 
+        **kwargs
+    ):
+        if config.data_dir:
+            config.data_dir = os.fspath(config.data_dir)  # Pathlib -> str
+        elif os.environ.get('TFDS_DIR', None):
+            config.data_dir = os.fspath(os.environ['TFDS_DIR'])
+        elif os.environ.get('GCS_DIR', None):
+            config.data_dir = os.fspath(os.environ['GCS_DIR'])
+        
+        self.dataset = dataset
+        file_format = config.file_format or file_adapters.DEFAULT_FILE_FORMAT
+        self._original_state = dict(data_dir = config.data_dir, config = config, version = config.version)
+        self._builder_config: BuilderConfig = self._create_builder_config(config)
+        self.__dict__['name'] = self._builder_config.name
+        self._version = self._pick_version(config.version)
+        self._data_dir_root, self._data_dir = self._build_data_dir(config)
+        
+        if config.rewrite_existing and gfile.Exists(self._data_dir):
+            self._remove_existing_data_dir()
+        
+        if gfile.Exists(self._data_dir):
+            self.info.read_from_directory(self._data_dir)
+            self._log_dataset_info()
+        else:
+            self.info.initialize_from_bucket()
+        
+        try:
+            self._file_format = file_adapters.FileFormat(file_format)
+            self.info.set_file_format(self._file_format)
+
+        except ValueError as e:
+            all_values = [f.value for f in file_adapters.FileFormat]
+            raise ValueError(f"{file_format} is not a valid format. Valid file formats: {all_values}") from e
+
+    def map_split_name(self, split: str):
+        """
+        Remaps Split Names to the ones used in the dataset
+        """
+        #if 'train' in split:
+        #    return _DataSplits['train']
+        if split in {'dev', 'develop', 'development', 'val', 'validation', 'eval', 'evaluation'}:
+            return _DataSplits['validation']
+        if split in {'test', 'testing'}:
+            return _DataSplits['test']
+        # assume train
+        return _DataSplits['train']
+
+
+    def _split_generators(
+        self, 
+        dl_manager: DownloadManager
+    ):
+        if isinstance(self.dataset, Dataset):
+            self.dataset = DatasetDict({'train': self.dataset})
+        return [
+            tfds.core.SplitGenerator(
+                name = self.map_split_name(split), 
+                gen_kwargs = {
+                    'dataset': self.dataset[split],
+                }
+            ) for split in self.dataset
+        ]
+
+
+    def _generate_examples(
+        self, 
+        dataset: Dataset,
+        **kwargs
+    ):
+        idx = 0
+        for ex in dataset:
+            for (n, i) in self.map_to_features(ex, idx, has_preprocessor = False):
+                yield n, i
+                idx += 1
+
+
+
 __all__ = [
     'Feature',
     'ShardInfo',
